package cn.com.infosec.openssl;

import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.ASN1Object;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.DERInteger;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.cms.ContentInfo;
import cn.com.infosec.asn1.pkcs.PrivateKeyInfo;
import cn.com.infosec.asn1.sec.ECPrivateKeyStructure;
import cn.com.infosec.asn1.x509.AlgorithmIdentifier;
import cn.com.infosec.asn1.x509.RSAPublicKeyStructure;
import cn.com.infosec.asn1.x509.SubjectPublicKeyInfo;
import cn.com.infosec.asn1.x9.X9ObjectIdentifiers;
import cn.com.infosec.jce.ECNamedCurveTable;
import cn.com.infosec.jce.PKCS10CertificationRequest;
import cn.com.infosec.jce.spec.ECNamedCurveParameterSpec;
import cn.com.infosec.util.encoders.Base64;
import cn.com.infosec.util.encoders.Hex;
import cn.com.infosec.x509.X509AttributeCertificate;
import cn.com.infosec.x509.X509V2AttributeCertificate;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.Reader;
import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.cert.CertificateFactory;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.security.spec.DSAPrivateKeySpec;
import java.security.spec.DSAPublicKeySpec;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.RSAPrivateCrtKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.StringTokenizer;

public class PEMReader extends BufferedReader {
   private final PasswordFinder pFinder;
   private final String provider;

   public PEMReader(Reader reader) {
      this(reader, (PasswordFinder)null, "INFOSEC");
   }

   public PEMReader(Reader reader, PasswordFinder pFinder) {
      this(reader, pFinder, "INFOSEC");
   }

   public PEMReader(Reader reader, PasswordFinder pFinder, String provider) {
      super(reader);
      this.pFinder = pFinder;
      this.provider = provider;
   }

   private byte[] readBytes(String endMarker) {
      StringBuffer var2;
      var2 = new StringBuffer.<init>();

      String var3;
      while((var3 = this.readLine()) != null && var3.indexOf(endMarker) == -1) {
         var2.append(var3.trim());
      }

      if (var3 != null) {
         return Base64.decode(var2.toString());
      } else {
         throw new IOException(endMarker + " not found");
      }
   }

   private PublicKey readRSAPublicKey(String endMarker) {
      ByteArrayInputStream var2;
      var2 = new ByteArrayInputStream.<init>(this.readBytes(endMarker));
      ASN1Sequence endMarker1 = (ASN1Sequence)(new ASN1InputStream(var2)).readObject();
      RSAPublicKeyStructure var8;
      var8 = new RSAPublicKeyStructure.<init>(endMarker1);
      RSAPublicKeySpec endMarker2;
      RSAPublicKeySpec var10000 = endMarker2 = new RSAPublicKeySpec;
      RSAPublicKeyStructure var10001 = var8;
      BigInteger var9 = var8.getModulus();
      var10000.<init>(var9, var10001.getPublicExponent());
      String var5 = "RSA";

      try {
         return KeyFactory.getInstance(var5, this.provider).generatePublic(endMarker2);
      } catch (NoSuchProviderException var3) {
         throw new IOException("can't find provider " + this.provider);
      } catch (Exception var4) {
         throw new IOException("problem extracting key: " + var4.toString());
      }
   }

   private PublicKey readPublicKey(String endMarker) {
      X509EncodedKeySpec var2;
      var2 = new X509EncodedKeySpec.<init>(this.readBytes(endMarker));
      byte endMarker1 = 2;
      String[] var3;
      String[] var10000 = var3 = new String[2];
      var10000[0] = "DSA";
      var10000[1] = "RSA";

      for(int var4 = 0; var4 < endMarker1; ++var4) {
         try {
            return KeyFactory.getInstance(var3[var4], this.provider).generatePublic(var2);
         } catch (NoSuchAlgorithmException var5) {
         } catch (InvalidKeySpecException var6) {
         } catch (NoSuchProviderException var7) {
            throw new RuntimeException("can't find provider " + this.provider);
         }
      }

      return null;
   }

   private X509Certificate readCertificate(String endMarker) {
      ByteArrayInputStream var2;
      var2 = new ByteArrayInputStream.<init>(this.readBytes(endMarker));
      String var10000 = "X.509";

      try {
         return (X509Certificate)CertificateFactory.getInstance(var10000, this.provider).generateCertificate(var2);
      } catch (Exception var3) {
         throw new IOException("problem parsing cert: " + var3.toString());
      }
   }

   private X509CRL readCRL(String endMarker) {
      ByteArrayInputStream var2;
      var2 = new ByteArrayInputStream.<init>(this.readBytes(endMarker));
      String var10000 = "X.509";

      try {
         return (X509CRL)CertificateFactory.getInstance(var10000, this.provider).generateCRL(var2);
      } catch (Exception var3) {
         throw new IOException("problem parsing cert: " + var3.toString());
      }
   }

   private PKCS10CertificationRequest readCertificateRequest(String endMarker) {
      try {
         return new PKCS10CertificationRequest(this.readBytes(endMarker));
      } catch (Exception var2) {
         throw new IOException("problem parsing cert: " + var2.toString());
      }
   }

   private X509AttributeCertificate readAttributeCertificate(String endMarker) {
      return new X509V2AttributeCertificate(this.readBytes(endMarker));
   }

   private ContentInfo readPKCS7(String endMarker) {
      StringBuffer var2;
      var2 = new StringBuffer.<init>();
      ByteArrayOutputStream var3;
      var3 = new ByteArrayOutputStream.<init>();

      String var4;
      while((var4 = this.readLine()) != null && var4.indexOf(endMarker) == -1) {
         var2.append(var4.trim().trim());
         Base64.decode(var2.substring(0, var2.length() / 4 * 4), var3);
         var2.delete(0, var2.length() / 4 * 4);
      }

      if (var2.length() == 0) {
         if (var4 != null) {
            ByteArrayInputStream var6;
            var6 = new ByteArrayInputStream.<init>(var3.toByteArray());

            try {
               return ContentInfo.getInstance((new ASN1InputStream(var6)).readObject());
            } catch (Exception var5) {
               throw new IOException("problem parsing PKCS7 object: " + var5.toString());
            }
         } else {
            throw new IOException(endMarker + " not found");
         }
      } else {
         throw new RuntimeException("base64 data appears to be truncated");
      }
   }

   private KeyPair readKeyPair(String type, String endMarker) {
      boolean var3 = false;
      String var4 = null;
      StringBuffer var5;
      var5 = new StringBuffer.<init>();

      String var6;
      while((var6 = this.readLine()) != null) {
         if (var6.startsWith("Proc-Type: 4,ENCRYPTED")) {
            var3 = true;
         } else if (var6.startsWith("DEK-Info:")) {
            var4 = var6.substring(10);
         } else {
            if (var6.indexOf(endMarker) != -1) {
               break;
            }

            var5.append(var6.trim());
         }
      }

      byte[] endMarker1 = Base64.decode(var5.toString());
      if (var3) {
         PasswordFinder var16;
         if ((var16 = this.pFinder) == null) {
            throw new IOException("No password finder specified, but a password is required");
         }

         char[] var19;
         if ((var19 = var16.getPassword()) == null) {
            throw new IOException("Password is null, but a password is required");
         }

         StringTokenizer var10001 = new StringTokenizer(var4, ",");
         var4 = var10001.nextToken();
         byte[] var25 = Hex.decode(var10001.nextToken());
         endMarker1 = PEMUtilities.crypt(false, this.provider, endMarker1, var19, var4, var25);
      }

      ByteArrayInputStream var20;
      var20 = new ByteArrayInputStream.<init>(endMarker1);
      ASN1Sequence endMarker2 = (ASN1Sequence)(new ASN1InputStream(var20)).readObject();
      Object var10;
      ASN1Sequence var10000;
      DERInteger endMarker3;
      ASN1Sequence var10002;
      BigInteger endMarker4;
      ASN1Sequence var10003;
      DERInteger var21;
      DERInteger var22;
      BigInteger var23;
      BigInteger var24;
      DERInteger var26;
      BigInteger var27;
      DERInteger var28;
      BigInteger var29;
      BigInteger var30;
      BigInteger var31;
      BigInteger var32;
      Object var33;
      Object var34;
      ASN1Sequence var35;
      if (type.equals("RSA")) {
         var10000 = endMarker2;
         var35 = endMarker2;
         var10002 = endMarker2;
         var10003 = endMarker2;
         ASN1Sequence var10004 = endMarker2;
         ASN1Sequence var10005 = endMarker2;
         ASN1Sequence var10006 = endMarker2;
         endMarker3 = (DERInteger)endMarker2.getObjectAt(1);
         var21 = (DERInteger)var10006.getObjectAt(2);
         var22 = (DERInteger)var10005.getObjectAt(3);
         var26 = (DERInteger)var10004.getObjectAt(4);
         var28 = (DERInteger)var10003.getObjectAt(5);
         DERInteger var7 = (DERInteger)var10002.getObjectAt(6);
         DERInteger var8 = (DERInteger)var35.getObjectAt(7);
         DERInteger var9 = (DERInteger)var10000.getObjectAt(8);
         var34 = var10 = new RSAPublicKeySpec;
         BigInteger var11 = endMarker3.getValue();
         var34.<init>(var11, var21.getValue());
         var34 = var33 = new RSAPrivateCrtKeySpec;
         endMarker4 = endMarker3.getValue();
         var23 = var21.getValue();
         var24 = var22.getValue();
         var27 = var26.getValue();
         var29 = var28.getValue();
         var30 = var7.getValue();
         var31 = var8.getValue();
         var32 = var9.getValue();
         var34.<init>(endMarker4, var23, var24, var27, var29, var30, var31, var32);
      } else {
         var10000 = endMarker2;
         var35 = endMarker2;
         var10002 = endMarker2;
         var10003 = endMarker2;
         endMarker3 = (DERInteger)endMarker2.getObjectAt(1);
         var21 = (DERInteger)var10003.getObjectAt(2);
         var22 = (DERInteger)var10002.getObjectAt(3);
         var26 = (DERInteger)var35.getObjectAt(4);
         var28 = (DERInteger)var10000.getObjectAt(5);
         var34 = var33 = new DSAPrivateKeySpec;
         var29 = var28.getValue();
         var30 = endMarker3.getValue();
         var31 = var21.getValue();
         var32 = var22.getValue();
         var34.<init>(var29, var30, var31, var32);
         var34 = var10 = new DSAPublicKeySpec;
         DERInteger var36 = var22;
         DERInteger var37 = var21;
         DERInteger var39 = endMarker3;
         endMarker4 = var26.getValue();
         var23 = var39.getValue();
         var24 = var37.getValue();
         var27 = var36.getValue();
         var34.<init>(endMarker4, var23, var24, var27);
      }

      KeyFactory var12 = KeyFactory.getInstance(type, this.provider);
      KeyFactory var38 = var12;
      PublicKey var13 = var12.generatePublic((KeySpec)var10);
      return new KeyPair(var13, var38.generatePrivate((KeySpec)var33));
   }

   private ECNamedCurveParameterSpec readECParameters(String endMarker) {
      return ECNamedCurveTable.getParameterSpec(((DERObjectIdentifier)ASN1Object.fromByteArray(this.readBytes(endMarker))).getId());
   }

   private KeyPair readECPrivateKey(String endMarker) {
      Exception var10000;
      label118: {
         boolean var10001;
         ECPrivateKeyStructure var34;
         try {
            var34 = new ECPrivateKeyStructure;
         } catch (ClassCastException var25) {
            var10001 = false;
            throw new IOException("wrong ASN.1 object found in stream");
         } catch (Exception var26) {
            var10000 = var26;
            var10001 = false;
            break label118;
         }

         ECPrivateKeyStructure var2 = var34;

         AlgorithmIdentifier var35;
         try {
            var34.<init>((ASN1Sequence)ASN1Object.fromByteArray(this.readBytes(endMarker)));
            var35 = new AlgorithmIdentifier;
         } catch (ClassCastException var23) {
            var10001 = false;
            throw new IOException("wrong ASN.1 object found in stream");
         } catch (Exception var24) {
            var10000 = var24;
            var10001 = false;
            break label118;
         }

         AlgorithmIdentifier endMarker1 = var35;

         DERObjectIdentifier var10002;
         ECPrivateKeyStructure var39;
         try {
            var39 = var2;
            var10002 = X9ObjectIdentifiers.id_ecPublicKey;
         } catch (ClassCastException var21) {
            var10001 = false;
            throw new IOException("wrong ASN.1 object found in stream");
         } catch (Exception var22) {
            var10000 = var22;
            var10001 = false;
            break label118;
         }

         DERObjectIdentifier var3 = var10002;

         PrivateKeyInfo var36;
         try {
            var35.<init>(var3, var39.getParameters());
            var36 = new PrivateKeyInfo;
         } catch (ClassCastException var19) {
            var10001 = false;
            throw new IOException("wrong ASN.1 object found in stream");
         } catch (Exception var20) {
            var10000 = var20;
            var10001 = false;
            break label118;
         }

         PrivateKeyInfo var33 = var36;

         SubjectPublicKeyInfo var38;
         try {
            var36.<init>(endMarker1, var2.getDERObject());
            var38 = new SubjectPublicKeyInfo;
         } catch (ClassCastException var17) {
            var10001 = false;
            throw new IOException("wrong ASN.1 object found in stream");
         } catch (Exception var18) {
            var10000 = var18;
            var10001 = false;
            break label118;
         }

         SubjectPublicKeyInfo var4 = var38;

         PKCS8EncodedKeySpec var40;
         try {
            var38.<init>(endMarker1, var2.getPublicKey().getBytes());
            var40 = new PKCS8EncodedKeySpec;
         } catch (ClassCastException var15) {
            var10001 = false;
            throw new IOException("wrong ASN.1 object found in stream");
         } catch (Exception var16) {
            var10000 = var16;
            var10001 = false;
            break label118;
         }

         PKCS8EncodedKeySpec endMarker2 = var40;

         X509EncodedKeySpec var41;
         try {
            var40.<init>(var33.getEncoded());
            var41 = new X509EncodedKeySpec;
         } catch (ClassCastException var13) {
            var10001 = false;
            throw new IOException("wrong ASN.1 object found in stream");
         } catch (Exception var14) {
            var10000 = var14;
            var10001 = false;
            break label118;
         }

         X509EncodedKeySpec var32 = var41;

         try {
            var41.<init>(var4.getEncoded());
         } catch (ClassCastException var11) {
            var10001 = false;
            throw new IOException("wrong ASN.1 object found in stream");
         } catch (Exception var12) {
            var10000 = var12;
            var10001 = false;
            break label118;
         }

         String var42 = "ECDSA";

         KeyFactory var43;
         try {
            var43 = KeyFactory.getInstance(var42, this.provider);
         } catch (ClassCastException var9) {
            var10001 = false;
            throw new IOException("wrong ASN.1 object found in stream");
         } catch (Exception var10) {
            var10000 = var10;
            var10001 = false;
            break label118;
         }

         KeyFactory var27 = var43;

         PKCS8EncodedKeySpec var10003;
         PublicKey var10004;
         KeyFactory var37;
         KeyPair var44;
         KeyPair var45;
         try {
            var44 = new KeyPair;
            var45 = var44;
            var37 = var27;
            var10003 = endMarker2;
            var10004 = var27.generatePublic(var32);
         } catch (ClassCastException var7) {
            var10001 = false;
            throw new IOException("wrong ASN.1 object found in stream");
         } catch (Exception var8) {
            var10000 = var8;
            var10001 = false;
            break label118;
         }

         PublicKey var28 = var10004;

         try {
            var45.<init>(var28, var37.generatePrivate(var10003));
            return var44;
         } catch (ClassCastException var5) {
            var10001 = false;
            throw new IOException("wrong ASN.1 object found in stream");
         } catch (Exception var6) {
            var10000 = var6;
            var10001 = false;
         }
      }

      Exception var29 = var10000;
      throw new IOException("problem parsing EC private key: " + var29);
   }

   public Object readObject() {
      String var1;
      do {
         if ((var1 = this.readLine()) == null) {
            return null;
         }

         if (var1.indexOf("-----BEGIN PUBLIC KEY") != -1) {
            return this.readPublicKey("-----END PUBLIC KEY");
         }

         if (var1.indexOf("-----BEGIN RSA PUBLIC KEY") != -1) {
            return this.readRSAPublicKey("-----END RSA PUBLIC KEY");
         }

         if (var1.indexOf("-----BEGIN CERTIFICATE REQUEST") != -1) {
            return this.readCertificateRequest("-----END CERTIFICATE REQUEST");
         }

         if (var1.indexOf("-----BEGIN NEW CERTIFICATE REQUEST") != -1) {
            return this.readCertificateRequest("-----END NEW CERTIFICATE REQUEST");
         }

         if (var1.indexOf("-----BEGIN CERTIFICATE") != -1) {
            return this.readCertificate("-----END CERTIFICATE");
         }

         if (var1.indexOf("-----BEGIN PKCS7") != -1) {
            return this.readPKCS7("-----END PKCS7");
         }

         if (var1.indexOf("-----BEGIN X509 CERTIFICATE") != -1) {
            return this.readCertificate("-----END X509 CERTIFICATE");
         }

         if (var1.indexOf("-----BEGIN X509 CRL") != -1) {
            return this.readCRL("-----END X509 CRL");
         }

         if (var1.indexOf("-----BEGIN ATTRIBUTE CERTIFICATE") != -1) {
            return this.readAttributeCertificate("-----END ATTRIBUTE CERTIFICATE");
         }

         if (var1.indexOf("-----BEGIN RSA PRIVATE KEY") != -1) {
            try {
               return this.readKeyPair("RSA", "-----END RSA PRIVATE KEY");
            } catch (Exception var2) {
               throw new IOException("problem creating RSA private key: " + var2.toString());
            }
         }

         if (var1.indexOf("-----BEGIN DSA PRIVATE KEY") != -1) {
            try {
               return this.readKeyPair("DSA", "-----END DSA PRIVATE KEY");
            } catch (Exception var3) {
               throw new IOException("problem creating DSA private key: " + var3.toString());
            }
         }

         if (var1.indexOf("-----BEGIN EC PARAMETERS-----") != -1) {
            return this.readECParameters("-----END EC PARAMETERS-----");
         }
      } while(var1.indexOf("-----BEGIN EC PRIVATE KEY-----") == -1);

      return this.readECPrivateKey("-----END EC PRIVATE KEY-----");
   }
}
